"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var view_1 = require("tns-core-modules/ui/core/view");
var types_1 = require("tns-core-modules/utils/types");
var index_common_1 = require("./index.common");
var observable_1 = require("tns-core-modules/data/observable");
__export(require("./index.common"));
function notifyForItemAtIndex(owner, nativeView, view, eventName, index) {
    var args = {
        eventName: eventName,
        object: owner,
        index: index,
        view: view,
        ios: undefined,
        android: nativeView
    };
    owner.notify(args);
    return args;
}
var CardStack = (function (_super) {
    __extends(CardStack, _super);
    function CardStack() {
        var _this = _super.call(this) || this;
        index_common_1.CLog(index_common_1.CLogTypes.info, 'CardStack constructor...');
        return _this;
    }
    Object.defineProperty(CardStack.prototype, "android", {
        get: function () {
            return this.nativeView;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CardStack.prototype, "adapter", {
        get: function () {
            return this.android.getAdapter();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CardStack.prototype, "topPosition", {
        get: function () {
            return this._cardStackLayoutManager.getTopPosition();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CardStack.prototype, "selectedIndex", {
        get: function () {
            return this.topPosition;
        },
        set: function (value) {
        },
        enumerable: true,
        configurable: true
    });
    CardStack.prototype.createNativeView = function () {
        if (this.options && this.visibleCount) {
            this.visibleCount = this.options.visibleCount;
        }
        var containerLayout = new android.widget.LinearLayout(this._context);
        containerLayout.setLayoutParams(new android.widget.LinearLayout.LayoutParams(android.widget.LinearLayout.LayoutParams.MATCH_PARENT, android.widget.LinearLayout.LayoutParams.MATCH_PARENT));
        containerLayout.setOrientation(android.widget.LinearLayout.HORIZONTAL);
        this._cardStackView = new com.yuyakaido.android.cardstackview.CardStackView(this._context);
        return this._cardStackView;
    };
    CardStack.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        this.notify({
            eventName: index_common_1.CardStackCommon.loadedEvent,
            object: this
        });
    };
    CardStack.prototype.initNativeView = function () {
        var _this = this;
        ensureCardStackAdapterClass();
        var pageListener = new com.yuyakaido.android.cardstackview.CardStackListener({
            onCardDragging: function (side, ratio) {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.draggingEvent,
                    object: observable_1.fromObject({
                        side: side.name(),
                        ratio: ratio
                    })
                });
            },
            onCardAppeared: function (view, position) {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.appearedEvent,
                    object: observable_1.fromObject({
                        view: view,
                        position: position
                    })
                });
            },
            onCardSwiped: function (direction) {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.swipedEvent,
                    object: observable_1.fromObject({
                        direction: String(direction),
                        position: _this.topPosition - 1
                    })
                });
            },
            onCardDisappeared: function (view, position) {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.disappearedEvent,
                    object: observable_1.fromObject({
                        view: view,
                        position: position
                    })
                });
            },
            onCardCanceled: function () {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.canceledEvent,
                    object: observable_1.fromObject({
                        position: _this.topPosition
                    })
                });
            },
            onCardRewound: function () {
                _this.notify({
                    eventName: index_common_1.CardStackCommon.rewoundEvent,
                    object: observable_1.fromObject({
                        position: _this.topPosition
                    })
                });
            }
        });
        var adapter = new CardStackAdapterClass(this.items, new WeakRef(this));
        this._cardStackLayoutManager = new com.yuyakaido.android.cardstackview.CardStackLayoutManager(this._context, pageListener);
        this._cardStackLayoutManager.setStackFrom(com.yuyakaido.android.cardstackview.StackFrom.Top);
        this._cardStackLayoutManager.setVisibleCount(this.visibleCount);
        this._cardStackLayoutManager.setTranslationInterval(8.0);
        this._cardStackLayoutManager.setScaleInterval(0.95);
        this._cardStackLayoutManager.setSwipeThreshold(0.15);
        this._cardStackLayoutManager.setMaxDegree(20.0);
        this._cardStackLayoutManager.setDirections(com.yuyakaido.android.cardstackview.Direction.HORIZONTAL);
        this._cardStackLayoutManager.setCanScrollHorizontal(true);
        this._cardStackLayoutManager.setCanScrollVertical(true);
        this._cardStackLayoutManager.setSwipeableMethod(com.yuyakaido.android.cardstackview.SwipeableMethod.AutomaticAndManual);
        this._cardStackLayoutManager.setOverlayInterpolator(new android.view.animation.LinearInterpolator());
        this._cardStackView.setAdapter(adapter);
        this._cardStackView.setLayoutParams(new android.widget.LinearLayout.LayoutParams(android.widget.LinearLayout.LayoutParams.MATCH_PARENT, android.widget.LinearLayout.LayoutParams.MATCH_PARENT));
        this._cardStackView.setLayoutManager(this._cardStackLayoutManager);
        this.refresh();
    };
    CardStack.prototype.refresh = function (event) {
        if (event === void 0) { event = null; }
        if (event) {
            if (event.action === 'add') {
                if (types_1.isNullOrUndefined(this.items) || !types_1.isNumber(this.items.length)) {
                    return;
                }
                if (!this.nativeView) {
                    return;
                }
                var adapter = this._cardStackView.getAdapter();
                if (adapter) {
                    adapter.notifyItemRangeInserted(this.items.length - event.addedCount, event.addedCount);
                }
            }
        }
        else {
            if (types_1.isNullOrUndefined(this.items) || !types_1.isNumber(this.items.length)) {
                return;
            }
            if (!this.nativeView) {
                return;
            }
            var adapter = this._cardStackView.getAdapter();
            if (adapter) {
                adapter.notifyDataSetChanged();
            }
        }
    };
    CardStack.prototype.next = function (direction) {
        if (direction === void 0) { direction = 'RIGHT'; }
        if (direction === 'RIGHT') {
            var setting = new com.yuyakaido.android.cardstackview.SwipeAnimationSetting.Builder()
                .setDirection(com.yuyakaido.android.cardstackview.Direction.Right)
                .setDuration(com.yuyakaido.android.cardstackview.Duration.Normal.duration)
                .setInterpolator(new android.view.animation.AccelerateInterpolator())
                .build();
            this._cardStackLayoutManager.setSwipeAnimationSetting(setting);
            this._cardStackView.swipe();
        }
        else {
            var setting = new com.yuyakaido.android.cardstackview.SwipeAnimationSetting.Builder()
                .setDirection(com.yuyakaido.android.cardstackview.Direction.Left)
                .setDuration(com.yuyakaido.android.cardstackview.Duration.Normal.duration)
                .setInterpolator(new android.view.animation.AccelerateInterpolator())
                .build();
            this._cardStackLayoutManager.setSwipeAnimationSetting(setting);
            this._cardStackView.swipe();
        }
    };
    CardStack.prototype.previous = function () {
        var setting = new com.yuyakaido.android.cardstackview.RewindAnimationSetting.Builder()
            .setDirection(com.yuyakaido.android.cardstackview.Direction.Bottom)
            .setDuration(com.yuyakaido.android.cardstackview.Duration.Normal.duration)
            .setInterpolator(new android.view.animation.DecelerateInterpolator())
            .build();
        this._cardStackLayoutManager.setRewindAnimationSetting(setting);
        this._cardStackView.rewind();
    };
    CardStack.prototype.onLayout = function (left, top, right, bottom) {
        index_common_1.CLog(index_common_1.CLogTypes.info, "onLayout...");
        view_1.View.layoutChild(this, this, 0, 0, right - left, bottom - top);
    };
    CardStack.prototype._getDataItem = function (index) {
        index_common_1.CLog(index_common_1.CLogTypes.info, "_getDataItem...");
        return this.items.getItem ? this.items.getItem(index) : this.items[index];
    };
    CardStack.prototype.onItemsChanged = function (data) {
        index_common_1.CLog(index_common_1.CLogTypes.info, "_onItemsChanged...");
        this.refresh();
    };
    return CardStack;
}(index_common_1.CardStackCommon));
exports.CardStack = CardStack;
var CardStackAdapterClass;
function ensureCardStackAdapterClass() {
    if (CardStackAdapterClass) {
        return;
    }
    var CardStackAdapter = (function (_super) {
        __extends(CardStackAdapter, _super);
        function CardStackAdapter(items, owner) {
            var _this = _super.call(this) || this;
            _this.items = [];
            _this.items = items;
            _this.owner = owner;
            return global.__native(_this);
        }
        CardStackAdapter.prototype.onCreateViewHolder = function (container, type) {
            var view = this.owner.get()._createTemplateView();
            if (view.parent !== this.owner.get()) {
                this.owner.get().addChild(view);
            }
            else {
                view.parent.android.removeView(view.android);
            }
            view.nativeView.setId(android.view.View.generateViewId());
            return new ViewHolder(view.nativeView);
        };
        CardStackAdapter.prototype.onBindViewHolder = function (holder, position) {
            var _this = this;
            if (!holder) {
                return;
            }
            this.owner.get().views.forEach(function (view) {
                if (holder.itemView === view.nativeView) {
                    var _args = notifyForItemAtIndex(_this.owner.get(), view ? view.nativeView : null, view, index_common_1.ITEMLOADING, position);
                    view.bindingContext = _this.owner.get().items.getItem(position);
                }
            });
        };
        CardStackAdapter.prototype.getItemCount = function () {
            var result;
            if (types_1.isNullOrUndefined(this.owner.get().items) || !types_1.isNumber(this.owner.get().items.length)) {
                result = this.owner ? this.owner.get()._childrenCount : 0;
            }
            else {
                result = this.owner ? this.owner.get().items.length : 0;
            }
            index_common_1.CLog(index_common_1.CLogTypes.info, "CardStackAdapter getCount result = " + result);
            return result;
        };
        return CardStackAdapter;
    }(android.support.v7.widget.RecyclerView.Adapter));
    CardStackAdapterClass = CardStackAdapter;
}
var ViewHolder = (function (_super) {
    __extends(ViewHolder, _super);
    function ViewHolder(view) {
        var _this = _super.call(this, view) || this;
        return global.__native(_this);
    }
    return ViewHolder;
}(android.support.v7.widget.RecyclerView.ViewHolder));
//# sourceMappingURL=index.android.js.map